<?php
/*
Plugin Name: KeyShield SSO Authentication
Version: 1.0.4
Plugin URI: http://www.keyshieldsso.com
Description: Authenticate users against KeyShield SSO server.
Author: TDP Ltd.
Author URI: http://www.keyshieldsso.com
License: free
*/


add_action('admin_menu', 'kshield_authentication_add_options_page');
$kshield_authentication_opt = get_option('kshield_authentication_options');
$kshield_configured = $kshield_authentication_opt['kshield_url'] != '';



require_once (ABSPATH . WPINC . '/registration.php');

// plugin hooks into authentication system
add_filter('authenticate', array('KShieldAuthentication', 'authenticate'), 2, 1);
add_filter('login_url', array('KShieldAuthentication', 'bypassReauth'));

if (!class_exists('KShieldAuthentication')) {
  class KShieldAuthentication {
  
	function attribute($userData, $name) {
		if ($userData["attributes"]) {
			$attributes = $userData["attributes"];
			$value = $attributes[$name];
			if (is_array($value) && !empty($value)) 
			{
				return $value[0];
			}
			else
			{
				return $value;
			}
		}
		return null;
	}
	
	function getClientIP() {
		if ( isset($_SERVER["REMOTE_ADDR"]) ) {
			return $_SERVER["REMOTE_ADDR"] ;
		} else if ( isset($_SERVER["HTTP_X_FORWARDED_FOR"]) ) {
			return $_SERVER["HTTP_X_FORWARDED_FOR"] ;
		} else if ( isset($_SERVER["HTTP_CLIENT_IP"]) ) {
			return $_SERVER["HTTP_CLIENT_IP"] ;
		}
		
		return null;
	}
	

	function kshieldObtainObject($kshieldUrl, $ip, $mailAttr, $gnAttr, $snAttr, $kshieldApiKey, $userAttr) {
		$askedAttr[$mailAttr] = null;
		$askedAttr[$gnAttr] = null;
		$askedAttr[$snAttr] = null;
		
		if (isset($userAttr)) {
			$userAttr = trim($userAttr);
			if (!empty($userAttr)) {
				$askedAttr[$userAttr] = null;
			}
			else {
				$userAttr = NULL;
			}
		}
		
		$attrsStr = implode(",", array_keys($askedAttr));
		
		$json_url = $kshieldUrl.'/api/userByIP/'.$ip."?type=json&attributes=".$attrsStr."&key=".$kshieldApiKey;
		$ch = curl_init( $json_url );
	
		$options = array(
		CURLOPT_RETURNTRANSFER => true,
		CURLOPT_HTTPHEADER => array('Accept: application/json')
		);
	
		curl_setopt_array( $ch, $options );
		$result =  curl_exec($ch);
		
		if ($result) {
			return json_decode($result,true);
		}
		return null;
	}
	
	function kshieldObtainId($obj, $userAttr) {
		if ($obj) {
			if(!is_null($userAttr) && $userAttr!='') {
				if ($obj["attributes"]) {
					$attrs = $obj["attributes"];
					if ($attrs[$userAttr]) {
						return $attrs[$userAttr];
					}
				}
			}
			else {
				if ($obj["screenName"]) {
					return $obj["screenName"];
				}
			}
		}
		return null;
	}
	
	function customOrDefault($custom, $default) {
		$retCustom = trim($custom);
		if ($retCustom =='') {
			return $default;
		}
		else {
			return $retCustom;
		}
	}
    
  function authenticate($user) {
    if(! function_exists('curl_version')) {
    	return new WP_Error("login_error", "KeyShield SSO integration doesn't work. It needs curl php extension module installed!");
	  }
	  if(! function_exists('json_decode')) {
	  	return new WP_Error("login_error", "KeyShield SSO integration doesn't work. It needs json php module installed!");
	  }
  	
	  global $kshield_authentication_opt, $_SERVER;
	  $method = $_SERVER['REQUEST_METHOD'];
	  
	  if ($method != 'GET') {
			return $user;
	  }
	  
	  if ( is_a($user, 'WP_User') ) {
	  	return $user;
	  }
      
	  if ($kshield_configured) {
			return false;
	  }
	  
	  if (!function_exists('get_userdatabylogin'))
			die("Could not load user data");
	  
  	$usernameAttribute = trim($kshield_authentication_opt['kshield_user_attr']);
	
	$attrMail = self::customOrDefault($kshield_authentication_opt['kshield_mail_attr'],"mail");
	$attrGn   = self::customOrDefault($kshield_authentication_opt['kshield_gn_attr'],"givenName");
	$attrSn   = self::customOrDefault($kshield_authentication_opt['kshield_sn_attr'],"sn");
	
	$kshieldObj = self::kshieldObtainObject($kshield_authentication_opt['kshield_url'] , self::getClientIP(), $attrMail, $attrGn, $attrSn, $kshield_authentication_opt['kshield_api_key'], $usernameAttribute);
	  
    $username = self::kshieldObtainId($kshieldObj, $usernameAttribute);
    if ($username) {
			$user = get_userdatabylogin($username);
			if ($user) {
				return new WP_User($user->ID);
			}
			else if ($kshield_authentication_opt["kshield_create_user"]) {
				$user_info = array();
				$user_info['user_login'] = $username;
				$user_info['user_pass']  = md5(''.microtime());
				
				$user_info['user_email'] = self::attribute($kshieldObj,$attrMail);
				if (empty($user_info['user_email'])) {
					return new WP_Error("login_error", "KeyShield SSO error: The user account cannot be created. The LDAP mail attribute is missing. Please contact your administrator.<br />You can try login manually with your username and password.");
				}
	
				$user_info['first_name'] = self::attribute($kshieldObj,$attrGn);
				$user_info['last_name'] = self::attribute($kshieldObj,$attrSn);
				
				if (!empty($user_info['first_name']) && !empty($user_info['last_name'])) {
					$user_info['display_name'] = $user_info['first_name'].' '.$user_info['last_name'];
					$user_info['user_nicename'] = sanitize_title($user_info['display_name']);
				}
				
				$role = $kshield_authentication_opt["kshield_account"];
				if (!$role)
				{
					$role = "Contributor";
				}
				
				$user_info['role'] = strtolower($role);
				
				wp_insert_user($user_info);
				
				$user = get_userdatabylogin($username);
				return new WP_User($user->ID);
			}
			else {
			  return new WP_Error("login_error", "You are authenticated by KeyShield SSO but your user account does not exist. If you are not automatically logged in, please login for first time manually with your username and password - your user account will be created automatically.");
			}
	  }
	  else {
			return false;
	  }
  }
	
	function bypassReauth($login_url) {
        $login_url = remove_query_arg('reauth', $login_url);
        return $login_url;
    }
      
  }
 }

//----------------------------------------------------------------------------
//		ADMIN OPTION PAGE FUNCTIONS
//----------------------------------------------------------------------------

function kshield_authentication_add_options_page() {
  if (function_exists('add_options_page')) {
    add_options_page('KeyShield SSO Authentication', 'KeyShield SSO Authentication', 8, basename(__FILE__), 'kshield_authentication_options_page');
  }
} 

function kshield_authentication_options_page() {
	if(! function_exists('curl_version')) {
		die('KeyShield SSO needs curl php module installed!');
	}
	if(! function_exists('json_decode')) {
		die('KeyShield SSO needs json php module installed!');
	}
	
	
  global $wpdb;
  
  // Setup Default Options Array
  $optionarray_def = array(
			   'kshield_url' => '',
  		   'kshield_api_key' => '',
  			 'kshield_user_attr' => '',
			 'kshield_mail_attr' => '',
			 'kshield_gn_attr' => '',
			 'kshield_sn_attr' => '',
			   'kshield_create_user' => FALSE,	
				'kshield_account' => 'Contributor'			   
			   );
  
  if (isset($_POST['submit']) ) {    
    // Options Array Update
    $optionarray_update = array (
				 'kshield_url' => $_POST['kshield_url'],
				 'kshield_api_key' => $_POST['kshield_api_key'],
				 'kshield_user_attr' => $_POST['kshield_user_attr'],
				 'kshield_mail_attr' => $_POST['kshield_mail_attr'],
				 'kshield_gn_attr' => $_POST['kshield_gn_attr'],
				 'kshield_sn_attr' => $_POST['kshield_sn_attr'],
				 'kshield_create_user' => $_POST['kshield_create_user'],
				 'kshield_account' => $_POST['kshield_account'],
				 );
    
    
    update_option('kshield_authentication_options', $optionarray_update);
  }
  
  // Get Options
  $optionarray_def = get_option('kshield_authentication_options');
  $kshield_account = $optionarray_def["kshield_account"];
  ?>
	<div class="wrap">
	<h2>KeyShield SSO Authentication Options</h2>
	<form method="post" action="<?php echo $_SERVER['PHP_SELF'] . '?page=' . basename(__FILE__); ?>&updated=true">
	<fieldset class="options">
	<table class="form-table" style="width:75em">
	   <tr valign="center">
	   <th scope="row">KeyShield SSO server URL <i>(e.g.&nbsp;http://127.0.0.1:8485)</i>:</th> 
	   <td><input type="text" name="kshield_url" id="kshield_url" value="<?php echo $optionarray_def['kshield_url']; ?>" size="40" /></td>
	   </tr>
	   
	   <tr valign="center">
	   <th scope="row">KeyShield SSO API Key:</th> 
	   <td><input type="text" name="kshield_api_key" id="kshield_api_key" value="<?php echo $optionarray_def['kshield_api_key']; ?>" size="40" /></td>
	   </tr>
	   
	   <tr valign="center"> 
	   <th scope="row">Auto create new user account?</th> 
	   <td width="15px"><input name="kshield_create_user" type="checkbox" id="kshield_create_user" value="1" <?php checked('1', $optionarray_def['kshield_create_user']); ?>" /></td>
	   </tr>
	   
	   <tr valign="center"> 	   
	   <td colspan="2"><i>Your KeyShield SSO server must provide at least mail attribute to enable this option. (optionaly: sn, givenName)</i><br /><i>In the configuration interface of KeyShield SSO server set <strong>Optional API attributes</strong> in the respective authentication connector to: <strong>mail</strong>, <strong>sn</strong> and <strong>givenName</strong>.</i></td>
	   </tr>
	   
	   <tr valign="center">
	   <th scope="row">Username attribute <i>(optional)</i>:</th> 
	   <td><input type="text" name="kshield_user_attr" id="kshield_user_attr" value="<?php echo $optionarray_def['kshield_user_attr']; ?>" size="40" /></td>
	   </tr>
	   
	   <tr valign="center">
	   <th scope="row">Custom e-mail attribute <font face="monospace">[mail]</font> <i>(optional)</i>:</th> 
	   <td><input type="text" name="kshield_mail_attr" id="kshield_mail_attr" value="<?php echo $optionarray_def['kshield_mail_attr']; ?>" size="40" /></td>
	   </tr>
	   
	   <tr valign="center">
	   <th scope="row">Custom given name attribute <font face="monospace">[givenName]</font> <i>(optional)</i>:</th> 
	   <td><input type="text" name="kshield_gn_attr" id="kshield_gn_attr" value="<?php echo $optionarray_def['kshield_gn_attr']; ?>" size="40" /></td>
	   </tr>
	   
	   <tr valign="center">
	   <th scope="row">Custom sure name attribute <font face="monospace">[sn]</font> <i>(optional)</i>:</th> 
	   <td><input type="text" name="kshield_sn_attr" id="kshield_sn_attr" value="<?php echo $optionarray_def['kshield_sn_attr']; ?>" size="40" /></td>
	   </tr>
	   
	   <tr valign="center"> 
	   <th scope="row">Set role of new account to:</th> 
	   
	   <td width="15px">
	   <select name="kshield_account" style="width:150px;">
	   	   <option value="Administrator" <?php if($kshield_account=="Administrator"){echo 'selected="selected"';}?> >Administrator</option>
		   <option value="Editor" <?php if($kshield_account=="Editor"){echo 'selected="selected"';}?> >Editor</option>
		   <option value="Author" <?php if($kshield_account=="Author"){echo 'selected="selected"';}?> >Author</option>
           <option value="Contributor" <?php if($kshield_account=="Contributor" || empty($kshield_account)){echo 'selected="selected"';}?> >Contributor</option>
           <option value="Subscriber" <?php if($kshield_account=="Subscriber"){echo 'selected="selected"';}?> >Subscriber</option>
	   </select>
	   </td>
	   
	   </tr>
	</table>
 
	</fieldset>
	<p />
	<div class="submit">
		<input type="submit" name="submit" value="<?php _e('Update Options') ?> &raquo;" />
	</div>
	</form>
<?php
}
?>